<?php

namespace App\domain\amadeus;

use App\domain\amadeus\traits\AmadeusParamsTraits;
use Illuminate\Support\Facades\Log;

abstract class AmadeusRequestInSeries
{
    use AmadeusParamsTraits;

    protected mixed $mid;

    public function __construct()
    {
        $amadeusHeaders = new AmadeusHeaders();
        ['nonce' => $nonce, 'utcTimestamp' => $timestamp, 'mid' => $this->mid, 'passSHA' => $passSHA] = $amadeusHeaders->getHeaders();
    }

    protected function createCurlRequest(): string|bool
    {
        $curl = curl_init();

        $caBundlePath = base_path('cacert.pem');

        curl_setopt_array($curl, array(
            CURLOPT_URL => $this->getToAddress(),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $this->generateXMLRequest(),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: text/xml; charset=utf-8',
                'SOAPAction: ' . $this->action() . ''
            ),
            CURLOPT_CAINFO => $caBundlePath,
        ));

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            Log::info('Curl error: ' . curl_error($curl));
        }

        curl_close($curl);

        return $response;
    }

    protected function xmlResponseBodyToArray($response): array
    {
        $xmlObject = simplexml_load_string($response);

        $body = $xmlObject->xpath('soap:Body');

        return json_decode(json_encode($body), true);
    }

    protected function xmlHeader(int $sequenceNumber, string $amadeusSessionId, string $amadeusSecurityToken): string
    {
        return '
            <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:ses="http://xml.amadeus.com/2010/06/Session_v3">
              <soap:Header>
                <ses:Session TransactionStatusCode="InSeries">
                  <ses:SessionId>' . $amadeusSessionId . '</ses:SessionId>
                  <ses:SequenceNumber>' . $sequenceNumber . '</ses:SequenceNumber>
                  <ses:SecurityToken>' . $amadeusSecurityToken . '</ses:SecurityToken>
                </ses:Session>
                <add:MessageID xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->mid . '</add:MessageID>
                <add:Action xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->action() . '</add:Action>
                <add:To xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->getToAddress() . '</add:To>
                <link:TransactionFlowLink xmlns:link="http://wsdl.amadeus.com/2010/06/ws/Link_v1" />
              </soap:Header>
        ';
    }

    abstract protected function generateXMLRequest(): string;

    abstract protected function action(): string;
}
