<?php

namespace App\domain\amadeus;

use Exception;

class DocIssuanceFlowAction
{
    protected QueryPNRRetrieveAction $pnrRetrieve;
    protected QueryDocIssuanceIssueTicketAction $docIssuance;
    protected QuerySecuritySignOut $securitySignOut;

    public function __construct()
    {
        $this->pnrRetrieve = new QueryPNRRetrieveAction;
        $this->docIssuance = new QueryDocIssuanceIssueTicketAction;
        $this->securitySignOut = new QuerySecuritySignOut;
    }

    /**
     * @throws Exception
     */
    public function execute(string $controlNumber): array
    {
        $pnrRetrieveResponse = $this->pnrRetrieve->execute($controlNumber);

        $amadeusSessionId = session("amadeus-session-id");
        $amadeusSecurityToken = session("amadeus-security-token");

        $docIssuanceResponse = $this->docIssuance->execute($amadeusSessionId, $amadeusSecurityToken);
        $securitySignOutResponse = $this->securitySignOut->execute($amadeusSessionId, $amadeusSecurityToken);

//        if (isset($securitySignOutResponse[0]['Security_SignOutReply']['processStatus']['statusCode'])) {
//            if ($securitySignOutResponse[0]['Security_SignOutReply']['processStatus']['statusCode'] != "P") {
//                throw new Exception("Security sign out unsuccessful");
//            }
//        } else {
//            throw new Exception("Security sign out unsuccessful");
//        }

        return [
            "pnrRetrieve" => $pnrRetrieveResponse[0],
            "docIssuance" => $docIssuanceResponse[0]
        ];
    }
}
