<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class createFOPAction extends AmadeusRequestInSeries
{
    use FileStorageTrait;
    protected string $payType;
    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 3;
    }

    public function execute(string $payType,  string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->payType = $payType;
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2.3-create_form_of_payment_response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "2.3-create_form_of_payment-response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in AddPNRMultiElementsAction execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error'=>$e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
        "
            <soap:Body>
                <FOP_CreateFormOfPayment>
                  <transactionContext>
                    <transactionDetails>
                      <code>FP</code>
                    </transactionDetails>
                  </transactionContext>
                  <fopGroup>
                    <fopReference />
                    <mopDescription>
                      <fopSequenceNumber>
                        <sequenceDetails>
                          <number>1</number>
                        </sequenceDetails>
                      </fopSequenceNumber>
                      <mopDetails>
                        <fopPNRDetails>
                          <fopDetails>
                            <fopCode>{$this->payType}</fopCode>
                          </fopDetails>
                        </fopPNRDetails>
                      </mopDetails>
                    </mopDescription>
                  </fopGroup>
                </FOP_CreateFormOfPayment>
            </soap:Body>
        </soap:Envelope>
        ";

        self::storeInFile("xml", "2.3-create_form_of_payment-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/TFOPCQ_19_2_1A";
    }
}
